unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  EditBtn, StdCtrls, Buttons, uRotateBitmap;

type

  { TForm1 }

  TForm1 = class(TForm)
    PaintBox2: TPaintBox;
    PaintBox3: TPaintBox;
    TransparentCheckBox: TCheckBox;
    FileNameEdit1: TFileNameEdit;
    PaintBox1: TPaintBox;
    procedure FileNameEdit1AcceptFileName(Sender: TObject; var Value: String);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox2Paint(Sender: TObject);
    procedure PaintBox3Paint(Sender: TObject);
    procedure TransparentCheckBoxChange(Sender: TObject);
  private
    { private declarations }
    FPicture: TPicture;
    FGlyph: TButtonGlyph;
    procedure DrawButtonStates(TopOffset, MaxSize: Integer);
    procedure DrawButtonStates(Glyph: TRotatedGlyph; TopOffset, MaxSize: Integer
      );
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

uses
  Math;

{ TForm1 }

procedure TForm1.FileNameEdit1AcceptFileName(Sender: TObject; var Value: String
  );
begin
  FPicture.LoadFromFile(Value);
  PaintBox1.Invalidate;
  PaintBox2.Invalidate;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FPicture := TPicture.Create;
  FGlyph := TButtonGlyph.Create;
  FileNameEdit1.InitialDir := ExtractFileDir(ParamStr(0));
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FGlyph.Destroy;
  FPicture.Destroy;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
var
  RightBitmap: TCustomBitmap;
  LeftBitmap: TCustomBitmap;
  MaxSize: Integer;
begin
  if FPicture.Graphic = nil then
    Exit;
  MaxSize := Max(FPicture.Graphic.Width, FPicture.Graphic.Height);

  //draw directly
  FPicture.Graphic.Transparent := TransparentCheckBox.Checked;
  PaintBox1.Canvas.Draw(0, 0, FPicture.Graphic);

  RightBitmap := CreateRotatedBitmap(FPicture.Graphic as TRasterImage, rdRight);
  RightBitmap.Transparent := TransparentCheckBox.Checked;
  PaintBox1.Canvas.Draw(0, MaxSize + 4, RightBitmap);

  LeftBitmap := CreateRotatedBitmap(FPicture.Graphic as TRasterImage, rdLeft);
  LeftBitmap.Transparent := TransparentCheckBox.Checked;
  PaintBox1.Canvas.Draw(0, (MaxSize * 2) + 8, LeftBitmap);

  //draw through buttonglyph
  FGlyph.Glyph.Assign(FPicture.Graphic);
  DrawButtonStates(0, MaxSize);

  FGlyph.Glyph.Assign(RightBitmap);
  DrawButtonStates(MaxSize + 4, MaxSize);

  FGlyph.Glyph.Assign(LeftBitmap);
  DrawButtonStates(MaxSize * 2 + 8, MaxSize);

  RightBitmap.Destroy;
  LeftBitmap.Destroy;
end;

procedure TForm1.PaintBox2Paint(Sender: TObject);
var
  RotatedBitmap: TRotatedBitmap;
  RotatedGlyph: TRotatedGlyph;
  MaxSize: Integer;
begin
  if FPicture.Graphic = nil then
    Exit;

  MaxSize := Max(FPicture.Graphic.Width, FPicture.Graphic.Height);

  RotatedBitmap := TRotatedBitmap.Create;
  RotatedBitmap.Bitmap.Assign(FPicture.Graphic);
  RotatedBitmap.Transparent := TransparentCheckBox.Checked;

  RotatedBitmap.Draw(PaintBox2.Canvas, 0, 0);


  RotatedBitmap.Direction := rdRight;
  RotatedBitmap.Draw(PaintBox2.Canvas, 0, MaxSize + 4);

  RotatedBitmap.Direction := rdLeft;
  RotatedBitmap.Draw(PaintBox2.Canvas, 0, MaxSize * 2 + 8);

  RotatedBitmap.Destroy;

  RotatedGlyph := TRotatedGlyph.Create;
  RotatedGlyph.Bitmap.Assign(FPicture.Graphic);
  RotatedGlyph.Transparent := TransparentCheckBox.Checked;

  DrawButtonStates(RotatedGlyph, 0, MaxSize);

  RotatedGlyph.Direction := rdRight;
  DrawButtonStates(RotatedGlyph, MaxSize + 4, MaxSize);

  RotatedGlyph.Direction := rdLeft;
  DrawButtonStates(RotatedGlyph, MaxSize * 2 + 8, MaxSize);

  RotatedGlyph.Destroy;
end;

procedure TForm1.PaintBox3Paint(Sender: TObject);
var
  RotatedText: TRotatedText;
begin
  //Exit;
  RotatedText := TRotatedText.Create;
  RotatedText.Text := Caption;

  RotatedText.Draw(PaintBox3.Canvas, 0, 0);

  //Exit;

  RotatedText.Direction := rdLeft;
  RotatedText.Draw(PaintBox3.Canvas, 0, 20);

  //Exit;

  RotatedText.Direction := rdRight;
  RotatedText.Draw(PaintBox3.Canvas, 20, 20);

  RotatedText.Destroy;
end;

procedure TForm1.TransparentCheckBoxChange(Sender: TObject);
begin
  PaintBox1.Invalidate;
  PaintBox2.Invalidate;
end;

procedure TForm1.DrawButtonStates(Glyph: TRotatedGlyph; TopOffset, MaxSize: Integer);
begin
  Glyph.State := bsUp;
  Glyph.Draw(PaintBox2.Canvas, MaxSize + 4, TopOffset);

  Glyph.State := bsDown;
  Glyph.Draw(PaintBox2.Canvas, MaxSize * 2 + 8, TopOffset);

  Glyph.State := bsDisabled;
  Glyph.Draw(PaintBox2.Canvas, MaxSize * 3 + 12, TopOffset);


  Glyph.State := bsExclusive;
  Glyph.Draw(PaintBox2.Canvas, MaxSize * 4 + 16, TopOffset);

  Glyph.State := bsHot;
  Glyph.Draw(PaintBox2.Canvas, MaxSize * 5 + 20, TopOffset);
end;


procedure TForm1.DrawButtonStates(TopOffset, MaxSize: Integer);
var
  R: TRect;
  P: TPoint;
begin
  P := Point(0,0);
  R.Bottom := PaintBox1.Height;
  R.Right := PaintBox1.Width;
  R.Top := TopOffset;

  R.Left := MaxSize + 4;
  FGlyph.Draw(PaintBox1.Canvas, R, P, bsUp, TransparentCheckBox.Checked, 0);

  R.Left := MaxSize * 2 + 8;
  FGlyph.Draw(PaintBox1.Canvas, R, P, bsDown, TransparentCheckBox.Checked, 0);

  R.Left := MaxSize * 3 + 12;
  FGlyph.Draw(PaintBox1.Canvas, R, P, bsDisabled, TransparentCheckBox.Checked, 0);

  R.Left := MaxSize * 4 + 16;
  FGlyph.Draw(PaintBox1.Canvas, R, P, bsExclusive, TransparentCheckBox.Checked, 0);

  R.Left := MaxSize * 5 + 20;
  FGlyph.Draw(PaintBox1.Canvas, R, P, bsHot, TransparentCheckBox.Checked, 0);
end;

initialization
  {$I unit1.lrs}

end.

